# NAME        Test name
# SOURCES     Sources to use when building
# LIBRARIES   Libraries to use when linking
# OUTPUT_DIR  Output directory in which to build target files
function(setup_unit_tests)
  cmake_parse_arguments(
    "UTEST"
    ""
    "NAME;OUTPUT_DIR"
    "SOURCES;LIBRARIES"
    ${ARGN}
  )

  if(NOT UTEST_NAME)
    if(ARGV0)
      set(UTEST_NAME "${ARGV0}")
    else()
      message(FATAL_ERROR "NAME is not set.")
    endif()
  endif()

  if(NOT UTEST_SOURCES)
    message(FATAL_ERROR "SOURCES is not set.")
  endif()

  if(NOT UTEST_OUTPUT_DIR)
    set(UTEST_OUTPUT_DIR "${DEFAULT_OUTPUT_DIR}")
  endif()

  if(BUILD_UNIT_TESTS)
    FetchContent_MakeAvailable(GoogleTest)
    add_executable("${UTEST_NAME}" ${UTEST_SOURCES})

    set_target_properties("${UTEST_NAME}" gmock gmock_main gtest gtest_main PROPERTIES
      ENABLE_RTTI ON
      ENABLE_EXCEPTIONS ON
      RUNTIME_OUTPUT_DIRECTORY "${UTEST_OUTPUT_DIR}"
      LIBRARY_OUTPUT_DIRECTORY "${UTEST_OUTPUT_DIR}"
      COMPILE_PDB_OUTPUT_DIRECTORY_DEBUG "${UTEST_OUTPUT_DIR}"
      COMPILE_PDB_OUTPUT_DIRECTORY_RELWITHDEBINFO "${UTEST_OUTPUT_DIR}"
    )

    target_link_libraries("${UTEST_NAME}" PRIVATE
      GTest::gtest_main
      ${UTEST_LIBRARIES}
    )

    setup_target_compile_definitions("${UTEST_NAME}")
    setup_target_compile_options("${UTEST_NAME}")
    setup_target_link_options("${UTEST_NAME}")

    add_test(
      NAME "${UTEST_NAME}"
      COMMAND "$<TARGET_FILE:${UTEST_NAME}>"
    )
  endif()
endfunction()
